/*
  Web Server

 Un web server che gestisce varie funzionalità della centralina dell'allarme di casa:
 using an Arduino Wiznet Ethernet shield.


 Circuit:
 * Ethernet shield 
 * sensore per temperatura e pressione DHT11
 * ...
 * 
 * 

 created xx Oct 2015
 by Ludwig Bargagli
 modified xx Xxx 201x
 by xxx xxx

 */


 /*
 * Il sistema di allarme è il modello SN-2800
 * 
 * Il sistema di allarme ha un'uscita per il collegamento alla sirena
 * Quando l'allarme entra in funzione, l'uscita della sirena fornisce 10,5V, se non è in funzione fornisce 0V
 * Quando la sirena entra in funzione, la corrente che passa è 6mA (0,006A). Siamo sicuri?
 * L'uscita della sirena in origine era collegata solo alla sirena interna.
 * Adesso è collegata in parallelo a:
 * 1 - sirena interna
 * 2 - sensore "H120 - Standard Voltage Sensor" (0v - 25v) per verificare se l'allarme sta suonando; collegato ad un pin analogico di Arduino: se l'allarme non suona il valore è zero, se suona è 190
 * 3 - telecomando wireless sirene esterne (lo devo ancora collegare)
 * 
 * Due fili sono collegati al led 1 del sistema di allarme:
 * se 'allarme non è attivato la tensione è zero
 * se l'allarme è attivato la tensione è 1,85v, su un ingrsso analogico di Arduino il valore rilevato è 385
 * perciò devo collegare così i due fili: filo bianco a GND, filo rosso ad un ingresso analogico
 * 
 * Sensore "AC Voltage Sensor" FZ1440 per verificare se in casa c'è corrente.
 * Devo collegarlo ad un ingresso analogico.
 * Se non c'è correnteAC il valore rilevato è 504-519.
 * Se c'è corrente AC il valore rilevato più alto (fino a 731) o più basso (fino a 9)
 *  
 * 
 * Sensore DHT11 per misurare la temperatura e la pressione
 * 
 * Sensore BMP180 per misurare la pressione atmosferica e la temperatura
 * 
 * Schermo LCd 20x4 I2C
 * 
 * Trasmettitore RF 433Mhz
 * 
 */

//Sirena allarme: 0 
//Sirena allarme: 460 -> quando suona la sirena la tensione è circa 10v, su un ingresso analogico di Arduino il valore rilevato è 460
//Corrente 220v: 446
//Corrente 220v: 0    -> Tra 450 e 550 non c'è tensione, al di fuori di questi valori c'è tensione
//Allarme On/Off: 0 
//Allarme On/Off: 398 -> se l'allarme è attivato la tensione del led è 1,85v, su un ingresso analogico di Arduino il valore rilevato è 385

 
// aggiungo le librerie che mi servono
#include <SPI.h>               // Per lo spi dell'ethernet shield
#include <Ethernet.h>          // Per l'ethernet shield
#include <Time.h>              // Per l'ora
#include <TimeAlarms.h>        // Per gli allarmi orari (lanciano una funzione ad un'ora prestabilita
#include <EthernetUdp.h>       // Per la connessione al server dell'ora
#include "DHT.h"               // Per il sensore per temperatura e pressione DHT11
#include <SimpleTimer.h>       // TImer per lanciare le funzioni ad intervalli di tempo prestabiliti
#include <avr/pgmspace.h>      // PROGMEM lib
#include <aREST.h>             // libreria per le chiamate REST da un client
#include <avr/wdt.h>           // watchdog - serve per la libreria aRest
//#include <SD.h>
#include <Wire.h>              // Comes with Arduino IDE - La uso per lo schermo LCD
#include <SFE_BMP180.h>        // Libreria per il sensore barometrico BMP180 - BMP085
#include <LiquidCrystal_I2C.h> // Get the LCD I2C Library here: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
#include <RCSwitch.h>          // Libreria per la comunicazione con il trasmettitore e ricevitore RF 433Mhz
#include <MemoryFree.h>        // Per controllare la memoria libera - IN PRODUZIONE VA COMMENTATA
#include <DS1307RTC.h>         // per il circuito Real Time Clock (RTC) per allineare l'ora (DS3231 AT24C32 IIC Precision Clock Module Storage)

/* impostazioni per il sensore DHT11 */
const PROGMEM int DHTTYPE = DHT11;   // DHT 11 oppure DHT22 - Tipo di modello del sensore della temnperatura
const PROGMEM int DHTPIN = 7;        // pin digitale al quale è collegato il sensore della temperatura
float temperaturaUmidita[3];    // array che utilizzo per restituire la temperatura (posizione 0), l'umidità (posizione 1) e la temperatura percepita (posizione 2)
// Initialize DHT sensor.
// Note that older versions of this library took an optional third parameter to
// tweak the timings for faster processors.  This parameter is no longer needed
// as the current DHT reading algorithm adjusts itself to work on faster procs.
DHT dht(DHTPIN, DHTTYPE);

/* impostazioni per i pin che controllano il sistema di allarme */
#define ALARM_ON_OFF_PIN 7        // pin analogico al quale è collegato il sensore di voltaggio DC (direttamenre il LED 1 del sistema di allarme) per sapere se l'allarme è attivo oppure no
#define ALARM_CAMERA_ON_OFF_PIN 9 // pin analogico al quale è collegato il sensore di voltaggio DC (direttamenre il LED 9 del sistema di allarme) per sapere se l'allarme è attivo oppure no
#define ALARM_SIREN_PIN 6         // pin analogico al quale è collegata l'uscita per la sirena del sistema di allarme
#define CORRENTE_220v_PIN 8       // pin analogico al quale è collegato il sensore per i 220v per capire se c'è corrente

/* impostazioni per l'ethernet shield e per la scheda SD */
// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
//const PROGMEM char macString[] = "DE:AD:BE:EF:FE:ED";
//const PROGMEM String macString = "DE:AD:BE:EF:FE:ED";
String macString = "DE:AD:BE:EF:FE:ED";

IPAddress ip(192, 168, 10, 151);
// Initialize the Ethernet server library
// with the IP address and port you want to use
// (port 80 is default for HTTP):
EthernetServer server(80);
// per Arduino Uno
// digital pins 4, 10, 11, 12, 13 
// per Arduino Mega:
// digital pins 4, 10, 50, 51, 52 - the hardware SS pin, 53, is not used to select either the W5100 or the SD card, but it must be kept as an output or the SPI interface won't work.
const PROGMEM int W5100_SD = 4;       // SD card Slave Select (Ethernet version 06)
const PROGMEM int W5100_SS = 10;      // SS (chip Slave Select) (Ethernet version 06)
//const PROGMEM int W5100_MOSI = 11;   // MOSI (master output, slave input)
//const PROGMEM int W5100_MISO = 12;   // MISO (master input, slave output)
//const PROGMEM int W5100_SCK  = 13;   // SCK or SCLK (serial clock) - SPICLOCK

/* impostazioni per la connessione al server dell'ora */
const PROGMEM unsigned int localPort = 8888;  // local port to listen for UDP packets
//IPAddress timeServer(192, 43, 244, 18);     // time.nist.gov NTP server
//IPAddress timeServer(193, 204, 114, 232);   // ntp1.inrim.it NTP server
//IPAddress timeServer(192, 168, 10, 11);     // local NTP server
char timeServer[] = "time.nist.gov";          // time.nist.gov NTP server
const PROGMEM int NTP_PACKET_SIZE= 48;        // NTP time stamp is in first 48 bytes of message
byte packetBuffer[ NTP_PACKET_SIZE];          // buffer to hold incoming/outgoing packets
time_t prevDisplay = 0;                       // when the digital clock was displayed
EthernetUDP Udp;                              // A UDP instance to let us send and receive packets over UDP
//unsigned long ntpSyncTime = 21600;            // sync to NTP server every "ntpSyncTime" seconds, set to 1 hour or more to be reasonable
//unsigned long ntpLastUpdate = 0;              // keep track of how long ago we updated the NTP server
unsigned int syncLatency = 2;                 // adjust the sync latency with computer NTP client in seconds 

/* impostazioni per sincronizzare l'ora con il circuito RTC (DS3231 AT24C32 IIC Precision Clock Module Storage) */
const long timeZoneOffset = 3600L; // timeZoneOffset = (Time Zone) * 3600L - CeT: (+1) * 3600L = 3600L - eg. (+8) * 3600L = 28800L for Taipei, Taiwan

/* impostazioni per il timer */
// There must be one global SimpleTimer object. More SimpleTimer objects can be created and run, although there is little point in doing so.
SimpleTimer timer;

/* impostazioni per il web client */
//const PROGMEM char serverName[] = "10.0.1.144";  // il mio pc locale

/* impostazioni per lo schermo LCd */
/* info:
   https://arduino-info.wikispaces.com/LCD-Blue-I2C
*/
// 20 character 4 line I2C Display
// Backpack Interface labelled "YwRobot Arduino LCM1602 IIC V1"
// Connect Vcc and Ground, SDA to A4, SCL to A5 on Arduino
// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address

/* impostazioni per il sensore barometrico BMP180 - BMP085 */
/* 
 Connect VCC of the BMP085 sensor to 3.3V (NOT 5.0V!)
 Connect GND to Ground
 Connect SCL to i2c clock - on '168/'328 Arduino Uno/Duemilanove/etc thats Analog 5
 Connect SDA to i2c data - on '168/'328 Arduino Uno/Duemilanove/etc thats Analog 4
 EOC is not used, it signifies an end of conversion
 XCLR is a reset pin, also not used here
 ---
 Hardware connections:
- (GND) to GND
+ (VDD) to 3.3V
(WARNING: do not connect + to 5V or the sensor will be damaged!)
You will also need to connect the I2C pins (SCL and SDA) to your
Arduino. The pins are different on different Arduinos:
Any Arduino pins labeled:  SDA  SCL
Uno, Redboard, Pro:        A4   A5
Mega2560, Due:             20   21
Leonardo:                   2    3
Leave the IO (VDDIO) pin unconnected. This pin is for connecting
the BMP180 to systems with lower logic levels such as 1.8V
*/
SFE_BMP180 pressure;                  // You will need to create an SFE_BMP180 object, here called "pressure"
#define ALTITUDE 20.0                 // Altitudine del sensore
float pressioneTemperatura[2];  // array che utilizzo per restituire la pressione atmosferica (posizione 0) e la temperatura (posizione 1)
 
/* contatore che compare in basso a destra nello schermo LCD */
int contatore = 0;

/* paginazione */
int pagina = 1; // Mi serve per sapere quale pagina (schermata) devo mostrare nello schermo LCD
#define PAGINATION_PIN 30     // pin digitale in input al quale è collegato il pulsante per cambiare pagina sullo schermo LCd
int paginationButtonState = 0;     // current state of the button
int lastpaginationButtonState = 0; // previous state of the button paginationButtonState - Serve per capire quando si preme il bottone per cambiare pagina

/* costanti per definire i limiti dei valori per fa scattare gli allarmi */
const PROGMEM int TEMPERATURA_SOGLIA_MIN = 0;
const PROGMEM int TEMPERATURA_SOGLIA_MAX = 40;
const PROGMEM int ALARM_SIREN_VAL_SOGLIA = 100;       // sopra questa soglia la sirena sta suonando perché manda circa 10v al voltmetro
const PROGMEM int CORRENTE_220V_VAL_SOGLIA_MIN = 450;
const PROGMEM int CORRENTE_220V_VAL_SOGLIA_MAX = 550;

/* impostazioni per la libreria REST */
aREST rest = aREST(); // Create aREST instance
//int test001; // Variables to be exposed to the API

/* variabili che servono per rispondere ad una chiamata REST (servono anche funzioni) */
/* variabili per indicare se è stato lanciato un allarme */
int sirenAlarmSent = 0;  // le use con aRest perciò devono essere int, non possono essere boolean
int ac220vAlarmSent = 0; // allora 0 = false e 1 = true

/* variabili per sapere se c'è stata una variazione nello stato del sistema di allarme */
// 0 = allarme disattivato
// 1 = allarme attivato
// 2 = allarme attivato meno che nella zona sicura (camera)
// 3 = attivazione sirena allarme (faccio suonare la sirena)
int currentAlarmStatus = 0;

/* impostazioni per il trasmettitore RF 433Mhz */
RCSwitch mySwitch = RCSwitch();









// setup **********************************************************************************************************************
void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  //while (!Serial) {
  //  ; // wait for serial port to connect. Needed for Leonardo only
  //}
  Serial.println(F("Inizio setup"));

  pinMode(PAGINATION_PIN, INPUT);
  //pinMode(4, OUTPUT);
  //digitalWrite(4, HIGH);

  //pinMode(W5100_SS,OUTPUT);
  //pinMode(W5100_SD,OUTPUT);
  //digitalWrite(W5100_SD,LOW);   // enable the SD card reader
  //digitalWrite(W5100_SD,HIGH);  // disable the SD card reader
  //digitalWrite(W5100_SS,LOW);   // enable the Ethernet shield
  //digitalWrite(W5100_SS,HIGH);  // disable the Ethernet shield

  // se i pin sono analogici non devo impostarli a input
//  pinMode(ALARM_ON_OFF_PIN, INPUT);
//  pinMode(ALARM_SIREN_PIN, INPUT);
//  pinMode(CORRENTE_220v_PIN, INPUT); 

  Ethernet.begin(mac, ip); // start the Ethernet connection and the server:
  server.begin();          // start the server
  dht.begin();             // start dht sensor
  Udp.begin(localPort);    // start UDP
  lcd.begin(20,4);         // initialize the lcd for 20 chars 4 lines, turn on backlight

  // Initialize the barometric sensor (it is important to get calibration values stored on the device).
  if (pressure.begin()) {
    //Serial.println(F("BMP180 init success"));
  } else
  {
    //Serial.println(F("BMP180 init NOT success"));
    // Oops, something went wrong, this is usually a connection problem,
    // see the comments at the top of this sketch for the proper connections.
    //Serial.println(F("BMP180 init fail\n\n"));
    //while(1); // Pause forever.
  }

  // ------- Quick 3 blinks of backlight  -------------
  for(int i = 0; i< 2; i++)
  {
    lcd.backlight();
    delay(250);
    lcd.noBacklight();
    delay(250);
  }
  lcd.backlight(); // finish with backlight on
    
  //Serial.print("server is at ");
  //Serial.println(Ethernet.localIP());

  //-------- Write characters on the display ------------------
  // NOTE: Cursor Position: (CHAR, LINE) start at 0  
  lcd.setCursor(0,0); // Start at character 0 on line 0
  lcd.print(F("Ciao Ludo           "));
  delay(500);
  lcd.setCursor(0,1);
  lcd.print(F("server is at:       "));
  delay(500);
  lcd.setCursor(0,2);
  lcd.print(Ethernet.localIP());
  lcd.print(F("     "));
  delay(2000);

  /* sincronizzazione con il modulo RTC e con il server UTP per recuperare l'ora (DS3231 AT24C32 IIC Precision Clock Module Storage) */
  ntpSyncDS1307(); // aggiorna il modulo RTC con l'ora dal server NTP
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  // setSyncInterval(600); // 600 seconds (60*10) = 10 minuti - predefinito: 5 minuti

  
  /* sincronizzazione con il server UTP per recuperare l'ora */
  /*
  //Serial.println(F("Mi connetto per recuperare l'ora"));
  //Serial.println(F("waiting for sync"));
  setSyncProvider(getNtpTime);
  //Serial.println(F("Inizio while"));
  while(timeStatus()== timeNotSet)
  {
    // wait until the time is set by the sync provider
    //Serial.println(F("Dentro il while"));
    //Serial.println(F("timeStatus = "));
    lcd.setCursor(0,1);
    lcd.print(timeStatus());
    lcd.print(F("  wait...           "));
    //Serial.println(timeStatus());
    delay(1000);
    lcd.setCursor(0,1);
    lcd.print(timeStatus());
    lcd.print(F("  wait.............      "));
    setSyncProvider(getNtpTime);
  }
  lcd.setCursor(0,1);
  lcd.print(timeStatus());
  lcd.print(F("                   "));
  //Serial.println("Ora sincronizzata");
  lcd.setCursor(0,2);
  lcd.print(F("Ora sincronizzata      "));
  setSyncInterval(86400); // 86400 seconds (60*60*24) = 24 ore
  delay(500);
  */
  
  /*
   * timeNotSet= 0
   * timeNeedsSync= 1
   * timeSet= 2
   * 
   */

//  Serial.print("now(): ");
//  Serial.println(now());
//  Serial.print("hour(): ");
//  Serial.println(hour());
//  Serial.print("minute(): ");
//  Serial.println(minute());
//  Serial.print("second(): ");
//  Serial.println(second());
//  Serial.print("day(): ");
//  Serial.println(day());
//  Serial.print("month(): ");
//  Serial.println(month());
//  Serial.print("year(): ");
//  Serial.println(year());
//  time_t t = now();
//  Serial.print("hour(t): ");
//  Serial.println(hour(t));
//  Serial.print("minute(t): ");
//  Serial.println(minute(t));
//  Serial.print("second(t): ");
//  Serial.println(second(t));
//  Serial.print("day(t): ");
//  Serial.println(day(t));
//  Serial.print("month(t): ");
//  Serial.println(month(t));
//  Serial.print("year(t): ");
//  Serial.println(year(t));
  
  /*
  // Per trovare il valore di queste tre variabili
  Serial.println(F("timeNotSet= "));
  Serial.println(timeNotSet);
  Serial.println(F("timeNeedsSync= "));
  Serial.println(timeNeedsSync);
  Serial.println(F("timeSet= "));
  Serial.println(timeSet);
  Serial.println(F(" "));
  */

  // timed actions setup ------------------------------------------------------------------------------------------------------------------
  //timer.setInterval(15000, test01); // timer is triggered every 15 seconds
  //timer.setInterval(60000, test01); // timer is triggered every 60 seconds (60*1000)
  //timer.setInterval(30000, connectToServer);
  timer.setInterval(1000, gestioneContatore);    // 1 secondo
  //timer.setInterval(10000, updateLCdMeteo);    // 10 secondi
  timer.setInterval(10000, updateLCdOraData3);   // 10 secondi
  timer.setInterval(10000, updateLCDPrincipale); // 10 secondi
  timer.setInterval(600000, sendDataToServer);   // timer is triggered every 600 seconds, 10 minuti (600*1000=600000)
  //timer.setInterval(60000, sendDataToServer);  // timer is triggered every 60 seconds, 1 minuto (60*1000=60000)
  //timer.setInterval(30000, sendDataToServer);  // timer is triggered every 30 seconds, 1/2 minuto (30*1000=30000)
  timer.setInterval(2000, checkAlarmSensors);   // timer is triggered every 2 seconds

  //Alarm.alarmRepeat(19,55,0, testAlarm);  // 19:55pm every day
  //Alarm.alarmRepeat(00,01,0, azzeraAllarmiInviati);  // 00:01am every day

  /* impostazioni per la libreria REST */
  // Init variables and expose them to REST API
  //test001 = 24;
  rest.variable("sirenAlarmSent",&sirenAlarmSent);
  rest.variable("ac220vAlarmSent",&ac220vAlarmSent);
  rest.function("setAc220vAlarmSent",setAc220vAlarmSent);
  rest.function("setSirenAlarmSent",setSirenAlarmSent);
  rest.function("getAlarmStatus",getAlarmStatus);
  rest.function("getAlarmCorrente220vStatus",getAlarmCorrente220vStatus);
  rest.function("setAlarmStatus",setAlarmStatus);
  rest.function("getAlarmSirenStatus",getAlarmSirenStatus);
  // Give name and ID to device
  rest.set_id("001");
  rest.set_name("Ludo_Casa");

  // Start watchdog (viene usato per la libreria aRest)
  wdt_enable(WDTO_4S);


  /* impostazioni per il trasmettitore RF 433Mhz */
  mySwitch.enableTransmit(40);    // Transmitter is connected to Arduino Pin #40  
  //mySwitch.setPulseLength(320); // Optional set pulse length.
  //mySwitch.setProtocol(2);      // Optional set protocol (default is 1, will work for most outlets)
  mySwitch.setProtocol(1);  
  mySwitch.setRepeatTransmit(15); // Optional set number of transmission repetitions.

  Serial.println(F("Fine setup"));
} // setup()















// loop *******************************************************************************************************************************
void loop() {
  // this is where the "polling" occurs
  timer.run();
  
  listenEthernetConnections();
  //checkAlarmSensors();
  
  Alarm.delay(0);
  paginaUpdate();
//  Serial.print("pagina= ");
//  Serial.println(pagina);

//  int alarmCorrente220vPinVal = analogRead(CORRENTE_220v_PIN);
//  Serial.print("alarmCorrente220vPinVal= ");
//  Serial.println(alarmCorrente220vPinVal);

  
//  delay(1000);
} // loop()
















/* --- Inizio funzioni esposte dalla libreria aRest --- */

// Serve per impostare il valore della variabile ac220vAlarmSent
// va chiamata così: http://192.168.10.151/setAc220vAlarmSent?params=1
int setAc220vAlarmSent(String value) {
  //Serial.println(F("setAc220vAlarmSent()"));
  int intValue = value.toInt();
  ac220vAlarmSent = intValue;
  return 1;
}

// Serve per impostare il valore della variabile sirenAlarmSent
int setSirenAlarmSent(String value) {
  //Serial.println(F("setSirenAlarmSent()"));
  int intValue = value.toInt();
  sirenAlarmSent = intValue;
  return 1;
}

// Serve per impostatare l'allarme di casa
// parametro in input: String value
// 0 = allarme disattivato
// 1 = allarme attivato
// 2 = allarme attivato meno che nella zona sicura (camera)
// 3 = attivazione sirena allarme (faccio suonare la sirena)
// va chiamata così: http://192.168.10.151/setAlarmStatus?params=0
int setAlarmStatus(String value) {
  //Serial.println(F("setAlarmStatus()"));
  int intValue = value.toInt();
  //sirenAlarmSent = intValue;
  // Devo mandare un segnale RF a 433Mhz per impostare l'allarme (se non è già impostato nello stato richiesto)
  // leggo lo stato nel quale si trova l'allarme:
  int alarmOnOffVal = getAlarmStatus("0");
  if (intValue != alarmOnOffVal) {
    // devo inviare il segnale RF
    //...
  }
  // test:
  if ( intValue == 0 ) {
    // pulsante lucchetto aperto (allarme disattivato):
    //Serial.println("pulsante lucchetto aperto");
    mySwitch.send(5632048, 24);
  } else if ( intValue == 1 ) {
    // pulsante lucchetto chiuso (allarme attivato):
    //Serial.println("pulsante lucchetto chiuso");
    mySwitch.send(5632192, 24);
  } else if ( intValue == 2 ) {
    // pulsante lucchetto chiuso senza camera (allarme attivato senza camera):
    //Serial.println("pulsante lucchetto chiuso senza camera");
    mySwitch.send(5632003, 24);
  } else if ( intValue == 3 ) {
    // pulsante lucchetto suona allarme (attivazione sirena allarme):
    //Serial.println("pulsante lucchetto suona allarme");
    mySwitch.send(7554316, 24);
  } else if ( intValue > 3 ) {
    //Serial.println("valore di input erraro");
    return 0;
  }
   // sensore allarme studio:
   //mySwitch.send(2338753, 24);
   //Serial.println("sensore allarme sala");
   //mySwitch.send(5970625, 24);
   //Serial.println("sensore allarme cucina");
   //mySwitch.send(9186657, 24);
   //Serial.println("sensore allarme camera");
   //mySwitch.send(3818593, 24);
  return 1;
}

// Serve per sapere in quale stato è impostato l'allarme di casa
// parametro in output: return alarmValReturn
// 0 = allarme disattivato
// 1 = allarme attivato
// 2 = allarme attivato meno che nella zona sicura (camera)
// 3 = sta suonando la sirena di allarme (la sirena sta suonando)
// va chiamata così: http://192.168.10.151/getAlarmStatus?params=0
int getAlarmStatus(String value) {
  //Serial.println(F("getAlarmStatus()"));
  // int intValue = value.toInt();
  int alarmValReturn = 0;
  //int alarmOnOffPinVal = analogRead(ALARM_ON_OFF_PIN);
  //int alarmCameraOnOffPinVal = analogRead(ALARM_CAMERA_ON_OFF_PIN);
  int alarmOnOffPinVal = leggiAlarmOnOffPin(5);
  int alarmCameraOnOffPinVal = leggiAlarmCameraOnOffPin(5);
  int alarmSirenPinVal = analogRead(ALARM_SIREN_PIN);
  if (alarmOnOffPinVal > 100) {  // se l'allarme è attivato la tensione del led 1 è 1,85v, su un ingresso analogico di Arduino il valore rilevato è 385
      if (alarmCameraOnOffPinVal > 100) {
         alarmValReturn = 1;
      } else {
        alarmValReturn = 2;
      }
    } else {
      alarmValReturn = 0;
    }
  if( alarmSirenPinVal > ALARM_SIREN_VAL_SOGLIA ) {
    // L'allarme sta suonando
    alarmValReturn = 3;
  }  
  return alarmValReturn;
}

// Serve per sapere se in casa c'è la corrente AC a 220v
// restituisce 0 se c'è corrente, 1 se non c'è corrente
// va chiamata così: http://192.168.10.151/getAlarmCorrente220vStatus?params=0
int getAlarmCorrente220vStatus(String value) {
  //Serial.println(F("getAlarmCorrente220vStatus()"));
  boolean alarmCorrente220vPinVal = leggiSensoreCorrenteAC220v(5); // restituisce false se c'è corrente, true se non c'è corrente
  if (alarmCorrente220vPinVal) {
    return 0;
  } else {
    return 1;
  }
}

// Serve per sapere se in casa la sirena dell'allarme sta suonando
// restituisce 0 se la sirena non sta suonando, 1 se sta suonando
// va chiamata così: http://192.168.10.151/getAlarmSirenStatus?params=0
int getAlarmSirenStatus(String value) {
  //Serial.println(F("getAlarmCorrente220vStatus()"));
  int alarmSirenPinVal = analogRead(ALARM_SIREN_PIN);
  if( alarmSirenPinVal > ALARM_SIREN_VAL_SOGLIA) {
    return 1;
  } else {
    return 0;
  }
}


/* --- Fine funzioni esposte dalla libreria aRTest --- */



//void testAlarm() {
//  lcd.clear();
//  lcd.setCursor(0,2);
//  lcd.print(F("  "));
//  updateLCdOraData(2);
//}

void azzeraAllarmiInviati() {
  //Serial.println(F("azzeraAllarmiInviati()"));
  boolean sirenAlarmSent = 0;
  boolean ac220vAlarmSent = 0;
}

/* --- Inizio controllo dati da sensori sirena e corrente AC --- */

// Controlla se i dati rilevati da alcuni sensori sono nella norma, se non lo sono invia subito un messaggio con i dati al server
void checkAlarmSensors() {
  //Serial.println("checkAlarmSensors()");
  //Serial.print("-------------------------------------freeMemory()=");
  //Serial.println(freeMemory());
  int alarmSirenPinVal = analogRead(ALARM_SIREN_PIN);
  boolean alarmCorrente220vPinVal = leggiSensoreCorrenteAC220v(5);
  // Devo mettere gli if per lanciare l'allarme.
  // Se uno dei valori è fuori limite di sicurezza,
  // devo lanciare la funzione he fa la connessione al server e invia i valori
  // poi l'applicazione sul server fa i controlli su i valori e lancia l'allarme per email

//ALARM_SIREN_VAL_SOGLIA 100;
//CORRENTE_220V_VAL_SOGLIA_MIN 450;
//CORRENTE_220V_VAL_SOGLIA_MAX 550;

  if( (alarmSirenPinVal > ALARM_SIREN_VAL_SOGLIA) && !sirenAlarmSent==1) {
    // Se l'allarme sta suonando invia i valori al server per fargli inviare una email
    //Serial.println("ALARM: alarmSirenPinVal= " + alarmSirenPinVal);
    sendDataToServer();
    sirenAlarmSent = 1;
  } else if( alarmCorrente220vPinVal && !ac220vAlarmSent==1 ) { // alarmCorrente220vPinVal è false se c'è tensione, true se non c'è tensione // Tra 450 e 550 non c'è tensione, al di fuori di questi valori c'è tensione
    // Se manca la corrente invia i valori al server per fargli inviare una email
    //Serial.println(F("ALARM: alarmCorrente220vPinVal"));
    sendDataToServer();
    ac220vAlarmSent = 1;
  }
  // devo controllare se l'allarme viene attivsato e disattivato e mandare questa infomrazione al server che poi la manda a eventuali dispositivi di segnalazione
  // 0 = allarme disattivato
  // 1 = allarme attivato
  // 2 = allarme attivato meno che nella zona sicura (camera)
  // 3 = attivazione sirena allarme (faccio suonare la sirena)
  int currentAlarmStatusTemp = currentAlarmStatus;
  currentAlarmStatus = getAlarmStatus("0");
  //Serial.print(F("currentAlarmStatusTemp= "));
  //Serial.println(currentAlarmStatusTemp);
  //Serial.print(F("currentAlarmStatus= "));
  //Serial.println(currentAlarmStatus);
  if ( currentAlarmStatus != currentAlarmStatusTemp) {
    sendDataToServer("cambioStatoAllarme", currentAlarmStatus);
    //currentAlarmStatus = currentAlarmStatusTemp;
  }
} // void checkAlarmSensors()

// verifica se l'allarme è attivato facendo numberOfReadings letture successive
// se l'allarme non è attivato restituisce 0
// se l'allarme è attivato restituisce 400
int leggiAlarmOnOffPin(int numberOfReadings) {
  int countOn = 0;    // numero di volte che l'allarme è On: la lettura è maggiore di 100
  int countOff = 0;   // numero di volte che l'allarme è Off: la lettura è minore di 100
  int alarmOnOffPinVal = 0;
  for (int i=0; i < numberOfReadings; i++) {
    alarmOnOffPinVal = analogRead(ALARM_ON_OFF_PIN);
//    Serial.print("alarmOnOffPinVal= ");
//    Serial.println(alarmOnOffPinVal);
    delay(10);
    if (alarmOnOffPinVal > 100) {  // se l'allarme è attivato la tensione del led 1 è 1,85v, su un ingresso analogico di Arduino il valore rilevato è 385
      countOn++;
    } else {
      countOff++;
    }
  } // for
  if ( countOn >= countOff) {
     return 400;
  } else {
     return 0;
  }
} // leggiAlarmOnOffPin(int numberOfReadings)

// verifica se l'allarme in camera è attivato facendo numberOfReadings letture successive
// se l'allarme in camera non è attivato restituisce 0
// se l'allarme in camera è attivato restituisce 400
int leggiAlarmCameraOnOffPin(int numberOfReadings) {
  int countOn = 0;    // numero di volte che l'allarme è On: la lettura è maggiore di 100
  int countOff = 0;   // numero di volte che l'allarme è Off: la lettura è minore di 100
  int alarmCameraOnOffPinVal = 0;
  for (int i=0; i < numberOfReadings; i++) {
    alarmCameraOnOffPinVal = analogRead(ALARM_CAMERA_ON_OFF_PIN);
//    Serial.print("alarmCameraOnOffPinVal= ");
//    Serial.println(alarmCameraOnOffPinVal);
    delay(10);
    if (alarmCameraOnOffPinVal > 100) {  // se l'allarme è attivato la tensione del led 1 è 1,85v, su un ingresso analogico di Arduino il valore rilevato è 385
      countOn++;
    } else {
      countOff++;
    }
  } // for
  if ( countOn >= countOff) {
     return 400;
  } else {
     return 0;
  }
} // leggiAlarmCameraOnOffPin(int numberOfReadings)

/* --- Fine controllo dati da sensori sirena e corrente AC --- */


void paginaUpdate() {
  //Serial.println(F("paginaUpdate()"));
  int numberOfPages = 2;
//  int paginationButtonState = 0;     // current state of the button
//  int lastpaginationButtonState = 0; // previous state of the button
  paginationButtonState = digitalRead(PAGINATION_PIN);
  /*
  int paginationButton = -1;
  int inChar=-1;
  if (Serial.available() > 0) {
    inChar = Serial.parseInt();
  }
  Serial.print("inChar= ");
  Serial.println(inChar);
  paginationButton = inChar;
  */
//  Serial.print("paginationButtoStaten= ");
//  Serial.println(paginationButtonState);
  if (paginationButtonState != lastpaginationButtonState) { // compare the buttonState to its previous state
    //Serial.println("cambiamento di stato del bottone");
    if ( paginationButtonState == HIGH){ // Se il bottone è premuto (if the state has changed) // if the current state is HIGH then the button wend from off to on
      //Serial.println("stato del bottone = HIGH");
      if (pagina < numberOfPages ) {
        pagina++;
      } else {
        pagina = 1;
      }
      //delay(2000);
      updateLCDPrincipale();
    } else { // if (paginationButtonState)
      //Serial.println("stato del bottone = LOW");
    }
    delay(50); // Delay a little bit to avoid bouncing
  } // if (paginationButtonState != lastpaginationButtonState)
  lastpaginationButtonState = paginationButtonState; // save the current state as the last state, for next time through the loop
} // paginaUpdate()


// contatore che compare in basso a destra nello schermo LCD
void gestioneContatore(){
  //Serial.println("gestioneContatore()");
  if (contatore == 9 ) {
     contatore = 0;
  } else {
    contatore++;
  }
  lcd.setCursor(19,3);
  lcd.print(contatore);
} // gestioneContatore()



/* --- Inizio funzioni per aggiornare lo schermo LCD --- */

void updateLCDPrincipale() {
  //Serial.println(F("updateLCDPrincipale()"));
  if (pagina==1) {
    updateLCdMeteo();
  } else if (pagina==2) {
    updateLCDAlarmSensors();
  }
} // updateLCDPrincipale()

// funzione che scrive sullo schermo LCd temperatura, umidità e pressione
void updateLCDAlarmSensors() {
  //Serial.println(F("updateLCDAlarmSensors()"));
  if (pagina==2) {
    int alarmOnOffPinVal = analogRead(ALARM_ON_OFF_PIN);
    int alarmSirenPinVal = analogRead(ALARM_SIREN_PIN);
    boolean alarmCorrente220vPinVal = leggiSensoreCorrenteAC220v(5);
    // devo scrivere i valori sullo schermo LCD
    lcd.setCursor(0,0); lcd.print(F("                    ")); lcd.setCursor(0,0);
    lcd.print(F("allarme: "));
    if (alarmOnOffPinVal > 100) {  // se l'allarme è attivato la tensione del led è 1,85v, su un ingresso analogico di Arduino il valore rilevato è 385
      lcd.print(F("On "));
    } else {
      lcd.print(F("Off"));
    }
    lcd.print(F(" ("));
    lcd.print(alarmOnOffPinVal);
    lcd.print(F(")"));
    lcd.setCursor(0,1); lcd.print(F("                    ")); lcd.setCursor(0,1);
    lcd.print(F("sirena: "));
    if (alarmSirenPinVal > 100) {  // quando suona la sirena la tensione è circa 10v, su un ingresso analogico di Arduino il valore rilevato è 460
      lcd.print(F("On "));
    } else {
       lcd.print(F("Off"));
    }
    lcd.print(F(" ("));
    lcd.print(alarmSirenPinVal);
    lcd.print(F(")"));
    lcd.setCursor(0,2); lcd.print(F("                    ")); lcd.setCursor(0,2);
    lcd.print(F("220v.: "));
    if ( !alarmCorrente220vPinVal ) {  // alarmCorrente220vPinVal è false se c'è tensione, true se non c'è tensione // Tra 450 e 550 non c'è tensione, al di fuori di questi valori c'è tensione
      lcd.print(F("On "));
    } else {
      lcd.print(F("Off"));
    }
    lcd.print(F(" ("));
    lcd.print(alarmCorrente220vPinVal);
    lcd.print(F(")"));    
  } // if (pagina==2)
}

// funzione che scrive sullo schermo LCd temperatura, umidità e pressione
void updateLCdMeteo() {
  //Serial.println(F("updateLCdMeteo()"));
  if (pagina==1) {
    leggiTemperaturaUmiditaDHT(&temperaturaUmidita[0]); // The Arduino-environment though didn't compile the code nkcelectronics suggested and aborted with an error (cannot convert pointer type), so I had to change the function-call to «getdat(&data[0])» and then it worked perfectly. It seems, as if you have to specificly hand over the address of the first element of the array.
    leggiPressione(pressioneTemperatura); // ho utilizzato due tecniche diverse per gestire l'aggiornamento degli array tramite funzioni
    lcd.setCursor(0,0); // lcd.setCursor(col, row)
    lcd.print(F("temp.: "));
    //lcd.print(temperaturaUmidita[0]);   // temperatura DHT11
    lcd.print(temperaturaUmidita[0],1);   // temperatura DHT11
    lcd.print(F("*C ("));
    //lcd.print(pressioneTemperatura[1]); // temperatura BMP180
    lcd.print(pressioneTemperatura[1],1); // temperatura BMP180
    lcd.print(F(")"));
    lcd.setCursor(0,1);
    lcd.print(F("umid.: "));
    lcd.print(temperaturaUmidita[1]);   // umidità DHT11
    lcd.print(F("%      "));
    lcd.setCursor(0,2);
    lcd.print(F("hPa: "));
    lcd.print(pressioneTemperatura[0]); // pressione BMP180
    lcd.print(F("       "));
    lcd.setCursor(0,1);
    //lcd.print(F("umid.: ")); // i primi tre caratteri di questa riga non li scrive, perciò li devo riscrivere. Deve capire quale è la casusa per risolvere il problema.
  } // if (pagina==0)
} //void updateLCdMeteo()

// richiama updateLCdOraData(3), serve per mettere la chiamata nel timer senza usare parametri
void updateLCdOraData3() {
  //Serial.println("updateLCdOraData3()");
  updateLCdOraData(3);
}

// funzione che scrive sullo schermo LCd l'ora e la data
void updateLCdOraData(int lineNumber) {
  //Serial.println(F("updateLCdOraData() - inizio"));
  // lineNumber = 3;
  lcd.setCursor(0,lineNumber);
  lcd.print(leggiOraData());
  lcd.print(F("    "));
  //Serial.println("updateLCdOraData() - fine");
}

/* --- Fine funzioni per aggiornare lo schermo LCD --- */



/* --- Inizio funzioni per leggere i dati dai sensori --- */

// funzione che legge la pressione atmosferica
void leggiPressione(float pressioneTemperaturaTmp[]) {
  //Serial.println(F("leggiPressione()"));
  // la devo chiamare così: leggiPressione(&pressioneTemperatura)
  char status;
  double T,P;
  //double p0,a;

  // If you want sea-level-compensated pressure, as used in weather reports,
  // you will need to know the altitude at which your measurements are taken.
  // We're using a constant called ALTITUDE in this sketch:
  
  //  Serial.println();
  //  Serial.print(F("provided altitude: "));
  //  Serial.print(ALTITUDE,0);
  //  Serial.println(F(" meters"));
  
  // If you want to measure altitude, and not pressure, you will instead need
  // to provide a known baseline pressure. This is shown at the end of the sketch.

  // You must first get a temperature measurement to perform a pressure reading.
  
  // Start a temperature measurement:
  // If request is successful, the number of ms to wait is returned.
  // If request is unsuccessful, 0 is returned.

  status = pressure.startTemperature();
  if (status != 0)
  {
    // Wait for the measurement to complete:
    delay(status);

    // Retrieve the completed temperature measurement:
    // Note that the measurement is stored in the variable T.
    // Function returns 1 if successful, 0 if failure.

    status = pressure.getTemperature(T);
    if (status != 0)
    {
      // Print out the measurement:
//      Serial.print(F("temperature: "));
//      Serial.print(T,2);
//      Serial.println(F(" *C"));
      
      // Start a pressure measurement:
      // The parameter is the oversampling setting, from 0 to 3 (highest res, longest wait).
      // If request is successful, the number of ms to wait is returned.
      // If request is unsuccessful, 0 is returned.

      status = pressure.startPressure(3);
      if (status != 0)
      {
        // Wait for the measurement to complete:
        delay(status);

        // Retrieve the completed pressure measurement:
        // Note that the measurement is stored in the variable P.
        // Note also that the function requires the previous temperature measurement (T).
        // (If temperature is stable, you can do one temperature measurement for a number of pressure measurements.)
        // Function returns 1 if successful, 0 if failure.

        status = pressure.getPressure(P,T);
        if (status != 0)
        {
//          // Print out the measurement:
//          Serial.print(F("absolute pressure:             "));
//          Serial.print(P,2);
//          Serial.print(F(" mb (hPA), "));
//          Serial.print(P*0.0009869,4);
//          Serial.print(F(" atm, "));
//          Serial.print(P*0.7500615,2);
//          Serial.print(F(" mmHg, "));
//          Serial.print(P*0.0295333727,2);
//          Serial.println(F(" inHg"));          

          // The pressure sensor returns abolute pressure, which varies with altitude.
          // To remove the effects of altitude, use the sealevel function and your current altitude.
          // This number is commonly used in weather reports.
          // Parameters: P = absolute pressure in mb, ALTITUDE = current altitude in m.
          // Result: p0 = sea-level compensated pressure in mb

//          p0 = pressure.sealevel(P,ALTITUDE); // we're at 1655 meters (Boulder, CO)
//          Serial.print(F("relative (sea-level) pressure: "));
//          Serial.print(p0,2);
//          Serial.print(F(" mb (hPA), "));
//          Serial.print(p0*0.0009869,4);
//          Serial.print(F(" atm, "));
//          Serial.print(p0*0.7500615,2);
//          Serial.print(F(" mmHg, "));
//          Serial.print(p0*0.0295333727,2);
//          Serial.println(F(" inHg"));

          // On the other hand, if you want to determine your altitude from the pressure reading,
          // use the altitude function along with a baseline pressure (sea-level or other).
          // Parameters: P = absolute pressure in mb, p0 = baseline pressure in mb.
          // Result: a = altitude in m.
//
//          a = pressure.altitude(P,p0);
//          Serial.print(F("computed altitude: "));
//          Serial.print(a,0);
//          Serial.println(F(" meters "));
        }
        else {
          //Serial.println("error retrieving pressure measurement\n");
        }
      }
      else {
        //Serial.println("error starting pressure measurement\n");
      }
    }
    else {
      //Serial.println("error retrieving temperature measurement\n");
      }
  }
  pressioneTemperaturaTmp[0] = P;
  pressioneTemperaturaTmp[1] = T;
} // void leggiPressione(...)

// funzione che utilizza il sensore DHT11 per leggere temperatura e umidità
void leggiTemperaturaUmiditaDHT(float *temperaturaUmiditaTmp) {
  //Serial.println(F("leggiTemperaturaUmiditaDHT()"));
  // la devo chiamare così: leggiTemperaturaUmiditaDHT(&temperaturaUmidita)
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();
  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);
  temperaturaUmiditaTmp[0] = t;
  temperaturaUmiditaTmp[1] = h;
  temperaturaUmiditaTmp[2] = hic;
} // void temperaturaUmidita(...)

// controlla il sensore per verificare se c'è corrente a 220v
// restituisce false se c'è corrente, true se non c'è corrente
boolean leggiSensoreCorrenteAC220v(int numberOfReadings) {
  //Serial.println(F("leggiSensoreCorrenteAC220v()"));
  int countOkMin = 0; // numero di volte che la lettura minore di 450
  int countOkMax = 0; // numero di volte che la lettura è maggiore di 550
  int countOk = 0;    // numero di volte che la lettura è minore di 450 e maggiore di 550 (cioè quando c'è tensione 220v AC)
  int countKo = 0;    // numero di volte che la lettura è NEL range 450 - 550 (cioè quando NON c'è tensione 220v AC)
  int alarmCorrente220vPinVal = 0;
  boolean returnValue = false;
  for (int i=0; i < numberOfReadings; i++) {
    alarmCorrente220vPinVal = analogRead(CORRENTE_220v_PIN);
//    Serial.print("alarmCorrente220vPinVal= ");
//    Serial.println(alarmCorrente220vPinVal);
    //delay(10);
    delay(5);
    if ( alarmCorrente220vPinVal > CORRENTE_220V_VAL_SOGLIA_MAX ){
      countOkMax++;
    } else if (alarmCorrente220vPinVal < CORRENTE_220V_VAL_SOGLIA_MIN ) {
      countOkMin++;
    } else {
      countKo++;
    }
  } // for
  countOk = countOkMax + countOkMin;
//  Serial.print("countOkMax= ");
//  Serial.println(countOkMax);
//  Serial.print("countOkMin= ");
//  Serial.println(countOkMin);
//  Serial.print("countOk= ");
//  Serial.println(countOk);
//  Serial.print("countKo= ");
//  Serial.println(countKo);
  if ( countKo > countOk ) {
    returnValue = true;
    //Serial.print(F("countOkMax= "));
    //Serial.println(countOkMax);
    //Serial.print(F("countOkMin= "));
    //Serial.println(countOkMin);
    //Serial.print(F("countOk= "));
    //Serial.println(countOk);
    //Serial.print(F("countKo= "));
    //Serial.println(countKo);   
  }
//  delay(5000);
  return returnValue;
}

  
/* --- Fine funzioni per leggere i dati dai sensori --- */ 


String leggiOraData(){
  //Serial.println(F("leggiOraData() - inizio"));
  String oraData = "";
  // digital clock display of the time
  oraData += printDigits(hour());
  oraData += ":";
  oraData += printDigits(minute());
  //oraData += ":";
  //oraData += printDigits(second());
  oraData += " ";
  oraData += printDigits(day());
  oraData += "/";
  oraData += printDigits(month());
  oraData += "/";
  oraData += year();
//  Serial.print("oraData= ");Serial.println(oraData);
  //Serial.println("leggiOraData() - fine");
  return oraData;
}

String printDigits(int digits){
  //Serial.println("printDigits()");
  // utility function for digital clock display: prints preceding leading 0
//  Serial.print("digits= ");Serial.println(digits);
//  if(digits < 10)
//  {
//   digits = '0' + digits;
//  }
  String digitsReturn = String(digits);
  if (digitsReturn.length()<2) {
    digitsReturn = "0" + digitsReturn;
  }
//  Serial.print("digits= ");Serial.println(digits);
  return digitsReturn;
}

String leggiOraDataSql(){
  String oraData = "";
  oraData += year();
  oraData += "-";
  oraData += printDigits(month());
  oraData += "-";
  oraData += printDigits(day());
  oraData += "%20";
  oraData += printDigits(hour());
  oraData += ":";
  oraData += printDigits(minute());
  oraData += ":";
  oraData += printDigits(second());
  return oraData;
}



/* --- Inizio funzioni per le connessioni --- */

// serve per chiamare questa funzione senza parametri (dal timer)
void sendDataToServer() {
  sendDataToServer("", 0);
}

// funzione da lanciare periodicamente per inviare i dati (le letture dei sensori) al server
void sendDataToServer(String alarmType, int alarmValue) {
  //Serial.println(F("++++++++++++++++++++ Connected to server Inizio ++++++++++++++++++++"));
  //Serial.print(F("alarmType:"));
  //Serial.println(alarmType);
  //Serial.print(F("alarmValue:"));
  //Serial.println(alarmValue);
  lcd.setCursor(0,3);
  lcd.print(F("Conness. al server "));
  String percorsoSulServer = F("/diy/logData.php");
  //byte serverLudo[] = { 10, 0, 1, 144 }; // pc Ludo ufficio
  byte serverLudo[] = { 192, 168, 10, 4 }; // Cubieboard
  String hostLudo = F("192.168.10.4");
  leggiTemperaturaUmiditaDHT(&temperaturaUmidita[0]); // The Arduino-environment though didn't compile the code nkcelectronics suggested and aborted with an error (cannot convert pointer type), so I had to change the function-call to «getdat(&data[0])» and then it worked perfectly. It seems, as if you have to specificly hand over the address of the first element of the array.
  leggiPressione(pressioneTemperatura); // ho utilizzato due tecniche diverse per gestire l'aggiornamento degli array tramite funzioni
  int alarmOnOffVal = analogRead(ALARM_ON_OFF_PIN);
  int alarmSirenVal = analogRead(ALARM_SIREN_PIN);
  //int corrente220vVal = analogRead(CORRENTE_220v_PIN);
  boolean corrente220vVal = leggiSensoreCorrenteAC220v(5); // false c'è corrente, true non c'è corrente

  // initialize the library instance:
  EthernetClient client;
  // attempt to connect, and wait a millisecond:
  //Serial.println(F("connecting to server..."));

  //Serial.println(client.connect(serverLudo, 80));
  if (client.connect(serverLudo, 80)) {
    //Serial.println(F("making HTTP request..."));
    // make HTTP GET request to ...
    //client.println("GET /user/test01.php?prova=arduinoTest HTTP/1.1");
    //client.println("HOST: 10.0.1.144");
    
    //String get_1 = F("GET /user/test01.php?idDispositivo=2&idSensore=1");
    String get_1 = F("GET ");
    get_1 += percorsoSulServer;
    get_1 += F("?");
    get_1 += F("idDispositivo=1");

    if ( alarmType == "cambioStatoAllarme" ) {
      get_1 += F("&cambioStatoAllarme=");
      get_1 += F("1");
      get_1 += F("&cambioStatoAllarmeVal=");
      get_1 += alarmValue;
    } else {
      get_1 += F("&temperaturaDHT11=");
      get_1 += temperaturaUmidita[0];
      get_1 += F("&umiditaDHT11=");
      get_1 += temperaturaUmidita[1];
      get_1 += F("&temperaturaPercepitaDHT11=");
      get_1 += temperaturaUmidita[2];    
      get_1 += F("&pressioneBMP180=");
      get_1 += pressioneTemperatura[0];    
      get_1 += F("&temperaturaBMP180=");
      get_1 += pressioneTemperatura[1];
      get_1 += F("&allarmeOnOff=");
      get_1 += alarmOnOffVal;
      get_1 += F("&allarmeSirena=");
      get_1 += alarmSirenVal;
      get_1 += F("&corrente220v=");
      get_1 += corrente220vVal;
    }
    
    // non ci possono essere spazi vuoti ma devo inserire %20 al posto di ogni spazio
    get_1 += F("&data=");
    get_1 += leggiOraDataSql();
    get_1 += F("&ip=");
    // get_1 += Ethernet.localIP();
    get_1 += displayIpAddress(Ethernet.localIP());
    get_1 += F("&informazioni=");
    get_1 += F("");
    get_1 += F("&macaddress=");
    get_1 += macString;
    


            
    get_1 += F(" HTTP/1.1");
    //String get_2 = F("HOST: 10.0.1.144");
    String get_2 = F("HOST: ");
    get_2 += hostLudo;

    //Serial.println(F("get_1="));
    //Serial.println(get_1);
    //Serial.println(F("get_2="));
    //Serial.println(get_2);
    lcd.setCursor(0,3);
    lcd.print(F("Invio dati...      "));
    client.println(get_1);
    client.println(get_2);
    
    client.println("Connection: close");
    client.println();
    client.stop();
  } else {
    //Serial.println(F("PROBLEMA making HTTP request..."));
  }
  // note the time of this connect attempt:
  //lastAttemptTime = millis();
  lcd.setCursor(0,3);
  lcd.print(F("Disconn. dal server"));
  // azzera i contatori degli allarmi inviati così se ci sono nuovi allarmi possono essere nuovamente segnalati
  azzeraAllarmiInviati();
  Serial.println(F("++++++++++++++++++++ Connected to server Fine ++++++++++++++++++++"));
}


// listen for incoming clients - funzione che fa da web server per le chiamate aRest
void listenEthernetConnections() {
  //Serial.println(F("listenEthernetConnections()"));
  EthernetClient client = server.available();
//  if (client.connected()) {
//    lcd.setCursor(0,3);
//    lcd.print(F("Conn. di un client "));
//  }
  rest.handle(client);
  wdt_reset();
}

/*
// funzione che fa da web server e restituisce una pagina web
void listenEthernetConnections() {
  // listen for incoming clients
  EthernetClient client = server.available();
  if (client) {
    Serial.println(F("++++++++++++++++++++ Client connected Inizio ++++++++++++++++++++"));
    lcd.setCursor(0,3);
    lcd.print(F("Conn. di un client "));
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        // Reading temperature or humidity takes about 250 milliseconds!
        // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
        float h = dht.readHumidity();
        // Read temperature as Celsius (the default)
        float t = dht.readTemperature();
        // Compute heat index in Celsius (isFahreheit = false)
        float hic = dht.computeHeatIndex(t, h, false);
        int alarmOnOffVal = analogRead(ALARM_ON_OFF_PIN);
        int alarmSirenVal = analogRead(ALARM_SIREN_PIN);
        int corrente220vValAnalog = analogRead(CORRENTE_220v_PIN);
        char c = client.read();
        Serial.write(c);
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");  // the connection will be closed after completion of the response
          client.println("Refresh: 600");  // refresh the page automatically every 5 sec
          client.println();
          client.println("<!DOCTYPE HTML>");
          client.println("<html>");
//          client.println("<br />");
          client.print("Humidity: ");
          client.print(h);
          client.print(" % ");
          client.println("<br />");
          client.print("Temperature: ");
          client.print(t);
          client.print(" *C ");
          client.println("<br />");
          client.print("Heat index: ");
          client.print(hic);
          client.print(" *C ");
          client.println("<br />");
          client.println("<br />");
          client.print("Sirena allarme: ");
          client.println(alarmSirenVal);
          client.println("<br />");
          client.print("Corrente 220v: ");
          client.println(corrente220vValAnalog);
          client.println("<br />");
          client.print("Allarme On/Off: ");
          client.println(alarmOnOffVal);
          client.println("<br />");
          
          Serial.print("Humidity: ");
          Serial.print(h);
          Serial.print(" %\t");
          Serial.print("Temperature: ");
          Serial.print(t);
          Serial.print(" *C ");
          Serial.print("Heat index: ");
          Serial.print(hic);
          Serial.println(" *C ");
          Serial.println(" ");
          Serial.print("allarme: ");
          Serial.print(alarmOnOffVal);
          Serial.println(" ");
          Serial.print("corrente 220v: ");
          Serial.print(corrente220vValAnalog);
          Serial.println(" ");

          if( now() != prevDisplay)
          {
            prevDisplay = now();
            Serial.println(leggiOraData());
            client.println(leggiOraData());
          }
          
          client.println("</html>");
          
          break;
        }
        if (c == '\n') {
          // you're starting a new line
          currentLineIsBlank = true;
        }
        else if (c != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
    }
    // give the web browser time to receive the data
    delay(100);
    // close the connection:
    client.stop();
    lcd.setCursor(0,3);
    lcd.print(F("Fine conn. client  "));
    Serial.println(F("++++++++++++++++++++ Client connected Fine ++++++++++++++++++++"));
  }
} // void listenEthernetConnections()
*/

/* --- Fine funzioni perle connessioni --- */


/* --- Inizio funzioni generiche di utilità --- */

//IPAddress property is an array. One simple way to display the IP address is as follows:
String displayIpAddress(IPAddress address)
{
  //Serial.println(F("displayIpAddress()"));
  return String(address[0]) + "." + 
          String(address[1]) + "." + 
          String(address[2]) + "." + 
          String(address[3]);
}

/* --- Fine funzioni generiche di utilità --- */

/* // DA CANCELLARE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 setSyncProvider(getNtpTime);
  //Serial.println(F("Inizio while"));
  while(timeStatus()== timeNotSet)
  {
    // wait until the time is set by the sync provider
    //Serial.println(F("Dentro il while"));
    //Serial.println(F("timeStatus = "));
    lcd.setCursor(0,1);
    lcd.print(timeStatus());
    lcd.print(F("  wait...           "));
    //Serial.println(timeStatus());
    delay(1000);
    lcd.setCursor(0,1);
    lcd.print(timeStatus());
    lcd.print(F("  wait.............      "));
    setSyncProvider(getNtpTime);
  }
  lcd.setCursor(0,1);
  lcd.print(timeStatus());
  lcd.print(F("                   "));
  //Serial.println("Ora sincronizzata");
  lcd.setCursor(0,2);
  lcd.print(F("Ora sincronizzata      "));
*/

/* Funzione che sincronizza il circuito Real Time Clock (RTC) per allineare l'ora (DS3231 AT24C32 IIC Precision Clock Module Storage) con il server NTP */
void ntpSyncDS1307() {
  int i = 0; // contatore per contare i tentativi di sincronizzazione con il server UTP
  const int maxConnectionAttemps = 6; 
  lcd.setCursor(0,0);
  lcd.print(F("Conn. server NTP    "));
  lcd.setCursor(0,1);
  lcd.print(F("                    "));
  lcd.setCursor(0,2);
  lcd.print(F("                    "));
  lcd.setCursor(0,3);
  lcd.print(F("                    "));
  //delay(500);
  while (i<maxConnectionAttemps) {
    lcd.setCursor(19,0);
    lcd.print(i);
    sendNTPpacket(timeServer); // send an NTP packet to a time server
     // wait to see if a replay is available
    delay(2000);
    if (Udp.parsePacket()) {
      // We've received a packet, read the data from it
      Udp.read(packetBuffer, NTP_PACKET_SIZE); // read the packet into the buffer
      // the timstamp starts at byte 40 of the received packet and is four bytes,
      // or two words, long. First, extract the two words:
      unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
      unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);
      // combine the four bytes (two words) into a long integer
      // this is NTP time (seconds since Jan 1 1900)
      unsigned long secsSince1900 = highWord << 16 | lowWord;
      // now convert NTP time into everyday time:
      // Unix time starts on Jan 1 1970. In seconds, that's 2208988800:
      const unsigned long seventyYears = 2208988800L;
      // substract seventy years:
      unsigned long epoch = secsSince1900 - seventyYears + timeZoneOffset + syncLatency;
      setTime(epoch);
      RTC.set(epoch);
      // ntpLastUpdate = now();
      // output time and "Sync OK" message every sync
      Serial.print(year());
      Serial.print('-');
      Serial.print(month());
      Serial.print('-');
      Serial.print(day());
      Serial.print(' ');
      Serial.print(hour());
      Serial.print(':');
      Serial.print(minute());
      Serial.print(':');
      Serial.print(second());
      Serial.print(' ');
      Serial.println("Sync OK");
      lcd.setCursor(0,1);
      lcd.print(timeStatus());
      lcd.print(F("                   "));
      //Serial.println("Ora sincronizzata");
      lcd.setCursor(0,2);
      lcd.print(F("Ora sincronizzata      "));
      //delay(1000);
      i = maxConnectionAttemps +1;
    } else {
      lcd.setCursor(0,1);
      lcd.print(timeStatus());
      lcd.print(F("                   "));
      Serial.println("Ora NON sincronizzata-------");
      lcd.setCursor(0,2);
      lcd.print(F("Ora NON             "));
      lcd.setCursor(0,3);
      lcd.print(F("sincronizzata       "));
      i++;
      delay(5000); 
    }
  } // while (i<maxConnectionAttemps)
} // void ntpSyncDS1307()

/*-------- NTP code ----------*/
/*
// serve per sincronizzare l'orologio di Arduino con il server NTP dell'ora esatta
unsigned long getNtpTime()
{
  //Serial.println(F("getNtpTime()"));
  sendNTPpacket(timeServer); // send an NTP packet to a time server
  delay(1000);
  if ( Udp.parsePacket() ) {
    Udp.read(packetBuffer,NTP_PACKET_SIZE); // read packet into buffer

    //the timestamp starts at byte 40, convert four bytes into a long integer
    unsigned long hi = word(packetBuffer[40], packetBuffer[41]);
    unsigned long low = word(packetBuffer[42], packetBuffer[43]);
    // this is NTP time (seconds since Jan 1 1900
    unsigned long secsSince1900 = hi << 16 | low;
    // Unix time starts on Jan 1 1970
    const unsigned long seventyYears = 2208988800UL;
    unsigned long epoch = secsSince1900 - seventyYears; // subtract 70 years
    return epoch;
  }
  return 0; // return 0 if unable to get the time
}
*/


// send an NTP request to the time server at the given address
//unsigned long sendNTPpacket(IPAddress address)
unsigned long sendNTPpacket(char* address)
{
  //Serial.println(F("sendNTPpacket(...)"));
  memset(packetBuffer, 0, NTP_PACKET_SIZE); // set all bytes in the buffer to 0
  //Serial.println(F("sendNTPpacket(...) - 00"));
  // Initialize values needed to form NTP request
  packetBuffer[0] = B11100011; // LI, Version, Mode
  packetBuffer[1] = 0; // Stratum
  packetBuffer[2] = 6; // Max Interval between messages in seconds
  packetBuffer[3] = 0xEC; // Clock Precision
  // bytes 4 - 11 are for Root Delay and Dispersion and were set to 0 by memset
  packetBuffer[12] = 49; // four-byte reference ID identifying
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;
  //Serial.println(F("sendNTPpacket(...) - 01"));
  // send the packet requesting a timestamp:
  Udp.beginPacket(address, 123); //NTP requests are to port 123
  //Serial.println(F("sendNTPpacket(...) - 02"));
  Udp.write(packetBuffer,NTP_PACKET_SIZE);
  //Serial.println(F("sendNTPpacket(...) - 03"));
  Udp.endPacket();
  //Serial.println(F("sendNTPpacket(...) - fine"));
}



