<?php
// error_reporting(E_ALL);
// ini_set('display_errors', 1);
// require_once 'session.php';

class Database
{
	private static $dbName         = "";
	private static $dbHost         = "";
	private static $dbUsername     = "";
	private static $dbUserPassword = "";
	
	private static $cont  = null;
	 
	public function __construct() {
		// This is the constructor of class Database. Since it is a static class, initialization of this class is not allowed.
		//To prevent misuse of the class, we use a "die" function to remind users.
		//die('Init function is not allowed');
	}
	 
	public static function connect() {
		// This is the main function of this class. It uses singleton pattern to make sure only one PDO connection exist across the whole application.
		// Since it is a static method. We use Database::connect() to create a connection.
		// One connection through whole application
		
		// leggo dal fiel di configurazione i parametri per accedere al database
		// Parse with sections
		$fileConf = "conf/conf.txt";
		$fileCorretto = file_exists($fileConf);
		if (!$fileCorretto) {
			$fileConf = "../".$fileConf;
		}
		$ini_array = parse_ini_file($fileConf, true);
// 		print_r($ini_array);
		self::$dbName         = $ini_array["database"]["dbName"];
		self::$dbHost         = $ini_array["database"]["dbHost"];
		self::$dbUsername     = $ini_array["database"]["dbUsername"];
		self::$dbUserPassword = $ini_array["database"]["dbUserPassword"];
		
		if ( null == self::$cont ) {
			try	{
				self::$cont =  new PDO( "mysql:host=".self::$dbHost.";"."dbname=".self::$dbName, self::$dbUsername, self::$dbUserPassword);
// 				self::$cont->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			}
			catch(PDOException $e) {
				die($e->getMessage());
			}
		}
		return self::$cont;
	}
	 
	public static function disconnect()	{
		// Disconnect from database. It simply sets connection to NULL. We need to call this function to close connection.
		self::$cont = null;
	}
	
	// Recupera dal database l'ultima configurazione impostata per il sensore passato in input
	public function getLastSensorState($nomeConfigurazioneSensore)	{
		$tabella = "configurazioni";
		$lastState = "";
		try {
			$conn = Database::connect ();
			// set the PDO error mode to exception
			$conn->setAttribute ( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
				
			// seleziono il più recente per ogni tipo:
			$sql = "SELECT idConfigurazioni, radioButtonId, radioButtonNome, configurazione, data, idUtente FROM $tabella";
			$sql .= " WHERE data in (SELECT MAX(data) FROM $tabella WHERE radioButtonId='$nomeConfigurazioneSensore' )";
			$sql .= " AND radioButtonId='$nomeConfigurazioneSensore'";
		
			/* Mi connetto al DB ed eseguo la query */
			$conn = Database::connect();
			// set the PDO error mode to exception
			$conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$stmt = $conn->prepare($sql);
			$stmt->execute();
			// set the resulting array to associative
			$resultSetFetch = $stmt->setFetchMode(PDO::FETCH_ASSOC);
			$results = $stmt->fetchAll();
		
// 			echo "numero risultati: " . count($results) . "<br>";
// 			print_r($results);
// 			echo "<br>";
			
			foreach($results as $result) { // $keyNumero => $valueArrayRiga) // C'è solo un risultato in $results
// 				print_r($result);
// 				echo "<br>";
				$configurazione = $result['configurazione']; // valore della configurazione: 0 oppure 1
// 				$radioButtonId = $result['radioButtonId']; // nome della configurazione, equivalente all'id del radio button della form delle configurazioni
				$lastState = $configurazione;
			}
			Database::disconnect ();
		} catch ( PDOException $e ) {
			echo $sql . "<br>" . $e->getMessage ();
		} // try catch
		return $lastState;
	} // public function getLastSensorState
	
	// Recupera dal database l'ultima configurazione impostata per tutti i sensori
	public function getLastSensorsState()	{
		$tabella = "configurazioni";
		$lastState = array();
		try {
			$conn = Database::connect ();
			// set the PDO error mode to exception
			$conn->setAttribute ( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
	
			// seleziono il più recente per ogni tipo:
			$sql = "SELECT idConfigurazioni, radioButtonId, radioButtonNome, configurazione, data, idUtente FROM $tabella";
			$sql .= " WHERE data in (SELECT MAX(data) FROM $tabella GROUP BY radiobuttonId)";
	
			/* Mi connetto al DB ed eseguo la query */
			$conn = Database::connect();
			// set the PDO error mode to exception
			$conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$stmt = $conn->prepare($sql);
			$stmt->execute();
			// set the resulting array to associative
			$resultSetFetch = $stmt->setFetchMode(PDO::FETCH_ASSOC);
			$results = $stmt->fetchAll();
	
// 			echo "numero risultati: " . count($results) . "<br>";
// 			print_r($results);
// 			echo "<br>";
// 			$results:
// 			Array ( [idConfigurazioni] => 18 [radioButtonId] => radio_luceStudio [radioButtonNome] => [configurazione] => 1 [data] => 2016-04-15 17:42:02 [idUtente] => 1 )
// 			Array ( [idConfigurazioni] => 19 [radioButtonId] => radio_luceCamera [radioButtonNome] => [configurazione] => 1 [data] => 2016-04-15 17:42:02 [idUtente] => 1 )
				
			foreach($results as $result) { // $keyNumero => $valueArrayRiga) 
// 				print_r($result);
// 				echo "<br>";
				$configurazione = $result['configurazione']; // valore della configurazione: 0 oppure 1
				$radioButtonId = $result['radioButtonId']; // nome della configurazione, equivalente all'id del radio button della form delle configurazioni
				$lastState[$radioButtonId] = $configurazione;
			}
			Database::disconnect ();
		} catch ( PDOException $e ) {
			echo $sql . "<br>" . $e->getMessage ();
		} // try catch
// 		print_r($lastState);
// 		echo "<br>";
// 		$lastState:
// 		Array ( [radio_luceStudio] => 1 [radio_luceCamera] => 1 [radio_luceSala] => 0 [radio_luceCucina] => 0
//              [radio_avvolgibileStudio] => 0 [radio_avvolgibileCamera] => 0 [radio_avvolgibileSala] => 1 [radio_avvolgibileCucina] => 0
//              [radio_portaGarage] => 0
//              [radio_allarmeStudio] => 0 [radio_allarmeCamera] => 0 [radio_allarmeSala] => 0 [radio_allarmeCucina] => 0  [radio_allarmeGarage] => 0
//              [radio_sensoreFumoCasa] => 0 [radio_sensoreFumoGarage] => 0 [radio_sensoreAllagamentoCasa] => 0 [radio_sensoreAllarmeSala] => 0 [radio_sensoreFugaGasCasa] => 0 [radio_sirenaAllarme] => 0 [radio_correnteCasa] => 0 )
		return $lastState;
	} // public function getLastSensorsState
	
	public function getAlarmStatus() {
		$status = false;
		$lastState = $this->getLastSensorsState();
		$status = $lastState['radio_allarmeStudio'] || $lastState['radio_allarmeCamera'] || $lastState['radio_allarmeSala'] || $lastState['radio_allarmeCucina'] || $lastState['radio_allarmeGarage'];
		return 	$status;
	}
	
	public function getSirenStatus() {
		$status = false;
		$lastState = $this->getLastSensorsState();
		$status = $lastState['radio_sirenaAllarme'];
		return 	$status;
	}
	
	public function getVoltageStatus() {
		$status = false;
		$lastState = $this->getLastSensorsState();
		$status = $lastState['radio_tensione'];
		return 	$status;
	}
	
	
	
	public function getRiscaldamentoStatus() {
		$status = $this->getSensorState(7);
		return $status;
	}
	
	public function getIrrigazioneStatus() {
		$status = $this->getSensorState(8);
		return $status;
	}
	
	// Recupera lo stato del sistema di riscaldamento, irrigazione ecc.
	// se il sitema è acceso restituisce true, se è spento restituisce false
	public function getSensorState($idDispositivo)	{
		$tabella = "programmazioneOraria";
		$lastState = false;
		// H => 24-hour format of an hour with leading zeros - 00 through 23
		// G => 24-hour format of an hour without leading zeros	- 0 through 23
		$oraAttuale = date("G"); // solo le ore senza minuti e secondi, ad esempio se sono le 11:42 restiruisce 11
		// w => Numeric representation of the day of the week - 0 (for Sunday) through 6 (for Saturday)
		// N => ISO-8601 numeric representation of the day of the week (added in PHP 5.1.0) -	1 (for Monday) through 7 (for Sunday)
		$giornoAttuale = date("w"); //
		try {
			$conn = Database::connect ();
			// set the PDO error mode to exception
			$conn->setAttribute ( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
	
			// seleziono il più recente per ogni tipo:
			$sql = "SELECT idprogrammazioneOraria, idDispositivo, giornoSettimana, orarioInizio FROM $tabella";
			$sql .= " WHERE idDispositivo=$idDispositivo";
			$sql .= " AND giornoSettimana=$giornoAttuale";
			$sql .= " AND orarioInizio='$oraAttuale:00'";
			$sql .= " ORDER BY idprogrammazioneOraria DESC";
			$sql .= " LIMIT 1";
// 			echo "sql = $sql";
// 			echo "<br>";
			/* Mi connetto al DB ed eseguo la query */
			$conn = Database::connect();
			// set the PDO error mode to exception
			$conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$stmt = $conn->prepare($sql);
			$stmt->execute();
			// set the resulting array to associative
			$resultSetFetch = $stmt->setFetchMode(PDO::FETCH_ASSOC);
			$results = $stmt->fetchAll();
// 			print_r($results);
// 			echo "<br>";
// 			echo "numero risultati: " . count($results);
// 			echo "<br>";

			if (count($results)) {
				$lastState = true;
			}
// 			foreach($results as $result) { // $keyNumero => $valueArrayRiga)
// 				print_r($result);
// 				echo "<br>";
// 			}
			Database::disconnect ();
		} catch ( PDOException $e ) {
			echo $sql . "<br>" . $e->getMessage ();
		} // try catch
		// 		print_r($lastState);
		// 		echo "<br>";
		// 		$lastState:
		// 		Array ( [radio_luceStudio] => 1 [radio_luceCamera] => 1 [radio_luceSala] => 0 [radio_luceCucina] => 0 [radio_avvolgibileStudio] => 0 [radio_avvolgibileCamera] => 0 [radio_avvolgibileSala] => 1 [radio_avvolgibileCucina] => 0 [radio_portaGarage] => 0 [radio_allarmeStudio] => 0 [radio_allarmeCamera] => 0 [radio_allarmeSala] => 0 [radio_allarmeCucina] => 0 [radio_sensoreAllarmeStudio] => 0 [radio_sensoreAllarmeCamera] => 0 [radio_sensoreAllarmeSala] => 0 [radio_sensoreAllarmeCucina] => 0 )
		return $lastState;
	} // public function getLastSensorsState
	
	
	
	
	
	
	
	
} // class
?>